/**
 * 
 */
package edu.uoregon.tau.perfexplorer.glue;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.NoSuchElementException;
import java.util.StringTokenizer;

import javax.swing.JFrame;

import edu.uoregon.tau.perfdmf.Application;
import edu.uoregon.tau.perfdmf.Experiment;
import edu.uoregon.tau.perfdmf.Trial;
import edu.uoregon.tau.perfdmf.View;
import edu.uoregon.tau.perfexplorer.client.PerfExplorerClient;
import edu.uoregon.tau.perfexplorer.client.PerfExplorerModel;
import edu.uoregon.tau.perfexplorer.common.RMISortableIntervalEvent;
import edu.uoregon.tau.perfexplorer.server.PerfExplorerServer;




/**
 * @author khuck
 *
 */
public class Utilities {
	public static Trial getTrialFromView (View v, String tName) {
		boolean message = false;
    PerfExplorerServer server = getServer();
    List<View> views = new ArrayList();
    views.add(v);
  	List<Trial> trials = server.getTrialsForView(views, false);//TODO: Is this metadata ever needed?
    for (Trial trial : trials) {
      System.out.println(trial);
      if (trial.getName().trim().equals(tName.trim())) {
        if (!trial.isXmlMetaDataLoaded()) {
          try {
      trial.loadXMLMetadata(server.getDB());
    } catch (SQLException e) {
      System.err.println("Error getting metadata for trial");
      e.printStackTrace();
    }
        }
        return trial;
      }
    }
		message = true;
    return null;
  }
	
  public static Trial getTrial (String aName, String eName, String tName) {
		boolean message = false;
        PerfExplorerServer server = getServer();
		List<Application> apps = server.getApplicationList();
        for (Application app : apps ) {
            if (app.getName().equals(aName)) {
            	//System.out.println("Found app");
            	List<Experiment> exps = server.getExperimentList(app.getID());
            	for (Experiment exp : exps) {
            		if (exp.getName().trim().equals(eName.trim())) {
                    	//System.out.println("Found exp");
            			List<Trial> trials = server.getTrialList(exp.getID(), false);//TODO: Is this metadata ever needed?
            			for (Trial trial : trials) {
            				if (trial.getName().trim().equals(tName.trim())) {
            					if (!trial.isXmlMetaDataLoaded()) {
            						try {
										trial.loadXMLMetadata(server.getDB());
									} catch (SQLException e) {
										System.err.println("Error getting metadata for trial");
										e.printStackTrace();
									}
            					}
            					return trial;
            				}
            			}
						System.out.println("Could not find trial: " + tName);
						message = true;
            		}
            	}
				if (!message)
					System.out.println("Could not find experiment: " + eName);
				message = true;
			}
        }
		if (!message)
		System.out.println("Could not find application: " + aName);
        return null;
	}

	public static List<View> getViews () {
		boolean message = false;

    PerfExplorerServer server = getServer();
    return server.getViews(0); 
  }

	public static List<View> getSubViews (View view) {
		boolean message = false;

    PerfExplorerServer server = getServer();
    return server.getViews(view.getID()); 
  }

	public static List<View> getAllViews () {
		boolean message = false;

    PerfExplorerServer server = getServer();
    return server.getAllSubViews(0); 
  }

	public static View getView (String name) {
		boolean message = false;

    PerfExplorerServer server = getServer();
    List<View> views = server.getAllSubViews(0); 
    View found = new View();
    for (View view : views ) {
        if (view.toString().equals(name)) {
          found = view;
          break;
        }
    }
    return found;
  }
	
  public static View getSubView (View view, String name) {
		boolean message = false;

    PerfExplorerServer server = getServer();
    List<View> views = server.getAllSubViews(view.getID()); 
    View found = new View();
    for (View subView : views ) {
        if (subView.toString().equals(name)) {
          found = subView;
          break;
        }
    }
    return found;
  }

  public static List<Trial> getTrialsForView(View view)
  {
		boolean message = false;

    PerfExplorerServer server = getServer();

    List<View> views = new ArrayList<View>();
    views.add(view);

    return server.getTrialsForTAUdbView(views); 
  }

	public static List<Trial> getTrialsForExperiment (String aName, String eName) {
		boolean message = false;
        PerfExplorerServer server = getServer();
		List<Application> apps = server.getApplicationList();
        for (Application app : apps ) {
            if (app.getName().equals(aName)) {
            	//System.out.println("Found app");
            	List<Experiment> exps = server.getExperimentList(app.getID());
            	for (Experiment exp : exps) {
            		if (exp.getName().trim().equals(eName.trim())) {
                    	//System.out.println("Found exp");
            			List<Trial> trials = server.getTrialList(exp.getID(),false);//TODO: Is this metadata ever needed?
            			for (Trial trial : trials) {
            				if (!trial.isXmlMetaDataLoaded()) {
            					try {
									trial.loadXMLMetadata(server.getDB());
								} catch (SQLException e) {
									System.err.println("Error getting metadata for trial");
									e.printStackTrace();
								}
            				}
            			}
						return trials;
            		}
            	}
				if (!message)
					System.out.println("Could not find experiment: " + eName);
				message = true;
			}
        }
		if (!message)
		System.out.println("Could not find application: " + aName);
        return null;
	}

	public static void  deleteTrial (Trial t) {
		boolean message = false;
    PerfExplorerServer server = getServer();
		try {
		t.deleteTrial(server.getDB(), t.getID());
		} catch (SQLException e) {}
	}

	public static List<Application> getApplications () {
		boolean message = false;
    PerfExplorerServer server = getServer();
		return server.getApplicationList();
	}
	
	public static List<Experiment> getExperimentsForApplication (String aName) {
		boolean message = true;
    PerfExplorerServer server = getServer();
		List<Application> apps = server.getApplicationList();
        for (Application app : apps ) {
            if (app.getName().equals(aName)) {
            	List<Experiment> exps = server.getExperimentList(app.getID());
				return exps;
			}
        }
		if (!message)
		System.out.println("Could not find application: " + aName);
        return null;
	}

	public static Trial getCurrentTrial () {
		Trial trial = PerfExplorerModel.getModel().getTrial();
		// should we check for a valid trial?
		if (trial == null) {
			System.out.println("Utilities.getCurrentTrial() failed: No trial selected.");
		}
		return trial;
	}

	private static PerfExplorerServer getServer() {
		return PerfExplorerServer.getServer(null, "", "");
	}

	private static PerfExplorerServer getServer(String configName, String tauHome) {
		String home = System.getProperty("user.home");
		String slash = System.getProperty("file.separator");
		String configFile = home + slash + ".ParaProf" + slash + "perfdmf.cfg." + configName;
		return PerfExplorerServer.getServer(configFile, tauHome, "");
	}

	public static JFrame getClient() {
		String home = System.getProperty("user.home");
		String slash = System.getProperty("file.separator");
		String configFile = home + slash + ".ParaProf" + slash + "perfdmf.cfg";
		JFrame frame = new PerfExplorerClient(true, configFile, true);
		frame.pack();
		frame.setVisible(true);
		return frame;
	}

	public static int setSession (String name) {
		try {
			PerfExplorerServer server = getServer(name, "");
			List<String> configNames = server.getConfigNames();
			for (int i = 0 ; i < server.getSessionCount() ; i++) {
				server.setConnectionIndex(i);
//				System.out.println(server.getConnectionString());
				String tmpname = (String)configNames.get(i);
				if (tmpname.equals(name)) {
//					System.out.println("selected: " + server.getConnectionString());
					return i;
				}
			}
		} catch (Exception e) {}
		return 0;
	}
	
	public static LinkedHashMap<String, Double> sortHashMapByValues(Map<String, Double> passedMap, boolean ascending) {
		List<String> mapKeys = new ArrayList<String>(passedMap.keySet());
		List<Double> mapValues = new ArrayList<Double>(passedMap.values());
		Collections.sort(mapValues);
		Collections.sort(mapKeys);

		if (!ascending) {
			Collections.reverse(mapValues);
		}

		LinkedHashMap<String, Double> someMap = new LinkedHashMap<String, Double>();
		for (Double val : mapValues) {
			for (String key : mapKeys) {
				if (passedMap.get(key).toString().equals(val.toString())) {
					passedMap.remove(key);
					mapKeys.remove(key);
					someMap.put(key, val);
					break;
				}
			}
		}
		return someMap;
	} 
	
	public static List<RMISortableIntervalEvent> getEventsForTrial(Trial trial, int metricIndex) {
		List<RMISortableIntervalEvent> events = null;
        PerfExplorerServer server = getServer();
        events = server.getEventList(trial.getID(), metricIndex);
		return events;
	}

	/**
	 * Shorten a function name
	 * @param longName the long function name
	 * @return the shortened function name
	 */
	public static String shortenEventName(String longName) {
		StringTokenizer st = new StringTokenizer(longName, "(");
		String shorter = null;
		try {
			shorter = st.nextToken();
			if (shorter.length() < longName.length()) {
				shorter = shorter + "()";
			}
		} catch (NoSuchElementException e) {
			shorter = longName;
		}
		longName = shorter;
		st = new StringTokenizer(longName, "[{");
		shorter = null;
		try {
			shorter = st.nextToken();
		} catch (NoSuchElementException e) {
			shorter = longName;
		}
		return shorter.trim();
	}


}
